/*
 * Copyright 2018 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.ait.lienzo.client.core.image;

import com.ait.lienzo.client.core.Context2D;
import com.ait.lienzo.test.LienzoMockitoTestRunner;
import com.google.gwt.dom.client.ImageElement;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(LienzoMockitoTestRunner.class)
public class ImageElementProxyTest {

    @Mock
    private ImageElement imageElement;

    @Mock
    private com.google.gwt.user.client.ui.Image imageWidget;

    @Mock
    private Context2D context;

    private ImageElementProxy tested;

    @Before
    public void init() {
        tested = new ImageElementProxy(imageWidget,
                                       imageElement);
    }

    @Test
    public void testDraw() {
        tested.draw(context);
        verify(context, times(1)).drawImage(eq(imageElement),
                                            eq(0d),
                                            eq(0d));
    }

    @Test
    public void testIsLoaded() {
        assertTrue(tested.isLoaded());
    }

    @Test
    public void testSize() {
        when(imageElement.getWidth()).thenReturn(15);
        when(imageElement.getHeight()).thenReturn(70);
        assertEquals(15, tested.getWidth());
        assertEquals(70, tested.getHeight());
    }

    @Test
    public void testDrawWithClipArea() {
        final ImageClipBounds clipBounds = new ImageClipBounds(0, 0, 5, 5, 10, 10);
        tested.draw(context,
                    clipBounds);
        verify(context, times(1)).drawImage(eq(imageElement),
                                            eq(0d),
                                            eq(0d),
                                            eq(5d),
                                            eq(5d),
                                            eq(0d),
                                            eq(0d),
                                            eq(10d),
                                            eq(10d));
    }

    @Test
    public void testDestroy() {
        tested.destroy();
        verify(imageWidget, times(1)).removeFromParent();
    }
}
